import os
import secrets
import datetime

from flask import Flask, session, request, redirect, abort, Response, url_for

from requests_oauthlib import OAuth2Session

def create_app(test_config=None):
	app = Flask(__name__)
	app.config['SECRET_KEY'] = secrets.token_hex(128)
	app.config.from_pyfile('default_config.py')
	if not test_config:
		app.config.from_pyfile('config.py', silent=True)
	else:
		app.config.from_mapping(test_config)
	# OAuth2Session.fetch_token verifies that the passed URIs scheme (the scheme
	# of request.url) is HTTPS. The way we deploy this app, request.url does not
	# reflect the actual request url, so we disable this check.
	os.environ['OAUTHLIB_INSECURE_TRANSPORT'] = '1'

	@app.route('/auth')
	def auth():
		try:
			timestamp = datetime.datetime.fromtimestamp(session['timestamp'])
			client_id = session['client_id']
			user_id = session['user_id']
			user_name = session['user_name']
			user_nickname = session['user_nickname']
			user_email = session['user_email']
			user_ldap_dn = session['user_ldap_dn']
			user_groups = session['user_groups']
		except (KeyError, OverflowError):
			session.clear()
			abort(401)
		if datetime.datetime.now() - timestamp > datetime.timedelta(days=2) or \
					client_id != request.headers['X-CLIENT-ID']:
			session.clear()
			abort(401)
		resp = Response('Ok', 200)
		resp.headers['OAUTH-USER-ID'] = user_id
		resp.headers['OAUTH-USER-NAME'] = user_name
		resp.headers['OAUTH-USER-NICKNAME'] = user_nickname
		resp.headers['OAUTH-USER-EMAIL'] = user_email
		resp.headers['OAUTH-USER-LDAP-DN'] = user_ldap_dn
		resp.headers['OAUTH-USER-GROUPS'] = ','.join(user_groups)
		return resp

	def get_oauth(**kwargs):
		return OAuth2Session(request.headers['X-CLIENT-ID'],
			redirect_uri=request.headers['X-REDIRECT-URI'], **kwargs)

	@app.route('/cookiecheck')
	def login_cookiecheck():
		if not session.get('cookies_enabled'):
			session.clear()
			session['cookies_enabled'] = True
			abort(400, description='Enable cookies and reload two times to continue')
		session.clear()
		return redirect(url_for('login', url=request.values.get('url', '/')))

	@app.route('/login')
	def login():
		client = get_oauth()
		url, state = client.authorization_url(app.config['OAUTH2_AUTH_URL'])
		session.clear()
		session['state'] = state
		parts = request.full_path.split('?rawurl=', 1)
		if len(parts) == 2:
			session['url'] = parts[1]
		else:
			session['url'] = request.values.get('url', '/')
		return redirect(url)

	@app.route('/callback')
	def callback():
		redirect_url = session.get('url', '/')
		if 'state' not in session:
			session.clear()
			session['cookies_enabled'] = True
			# Redirect to login_cookiecheck to prevent redirect loop when cookies are disabled
			return redirect(url_for('login_cookiecheck', url=redirect_url))
		state = session['state']

		client = get_oauth(state=state)
		client.fetch_token(app.config['OAUTH2_TOKEN_URL'],
			client_secret=request.headers['X-CLIENT-SECRET'],
			authorization_response=request.url, verify=(not app.debug))
		userinfo = client.get(app.config['OAUTH2_USERINFO_URL']).json()

		session.clear()
		session['timestamp'] = datetime.datetime.now().timestamp()
		session['client_id'] = request.headers['X-CLIENT-ID']
		session['user_id'] = userinfo['id']
		session['user_name'] = userinfo['name']
		session['user_nickname'] = userinfo['nickname']
		session['user_email'] = userinfo['email']
		session['user_ldap_dn'] = userinfo['ldap_dn']
		session['user_groups'] = userinfo['groups']
		return redirect(redirect_url)

	@app.route('/logout')
	def logout():
		session.clear()
		resp = Response('Ok', 200)
		if request.values.get('redirect_url'):
			resp = redirect(request.values.get('redirect_url'))
		if request.values.get('clear_cookies'):
			for key, _ in request.cookies.items():
				resp.delete_cookie(key)
		return resp

	@app.route('/status')
	def status():
		resp = Response(f'''Proxy Configuration Status

For this proxy service to work properly, the OAuth client crendentials must
be injected in by the webserver as HTTP-headers:

X-CLIENT-ID: {request.headers.get('X-CLIENT-ID', '(unset)')}
X-CLIENT-SECRET: {'(set)' if request.headers.get('X-CLIENT-SECRET') else '(unset)'}
X-REDIRECT-URI: {request.headers.get('X-REDIRECT-URI', '(unset)')}

If you accessed this ressource with the URL

		https://mydomain/mysubpath/info

then the redirect URI must be set to:

		https://mydomain/mysubpath/callback

This exact redirect URI must also be registered with the OAuth server as
a valid redirect_uri for the client_id.
''')
		resp.mimetype = 'text/plain; charset=utf-8'
		return resp

	return app
